Namespace Calculate

    ''' <completionlist cref="madrat.Calculate.Operation"></completionlist>
    Public Class Operation

#Region " Public "

        Public ReadOnly IsBinary As Boolean

#Region " Methods "

        Public Overrides Function ToString() As String
            Return Representation
        End Function

        Public Function Perform(ByVal ParamArray values As Double()) As Double
            Select Case Me
                Case Add
                    Return values(0) + values(1)
                Case Subtract
                    Return values(0) - values(1)
                Case Multipy
                    Return values(0) * values(1)
                Case Divide
                    Return values(0) / values(1)
            End Select
        End Function

#End Region

#Region " Static "

        Public Shared ReadOnly Add As New Operation("+", True)
        Public Shared ReadOnly Subtract As New Operation("-", True)
        Public Shared ReadOnly Multipy As New Operation("*", True)
        Public Shared ReadOnly Divide As New Operation("/", True)

        Public Shared Operator =(ByVal lhs As Operation, ByVal rhs As Operation) As Boolean
            Return String.Compare(lhs.Representation, rhs.Representation, True) = 0
        End Operator

        Public Shared Operator <>(ByVal lhs As Operation, ByVal rhs As Operation) As Boolean
            Return String.Compare(lhs.Representation, rhs.Representation, True) <> 0
        End Operator

        Public Shared Operator =(ByVal lhs As Operation, ByVal rhs As String) As Boolean
            Return String.Compare(lhs.Representation, rhs, True) = 0
        End Operator

        Public Shared Operator <>(ByVal lhs As Operation, ByVal rhs As String) As Boolean
            Return String.Compare(lhs.Representation, rhs, True) <> 0
        End Operator

        Public Shared Operator =(ByVal lhs As String, ByVal rhs As Operation) As Boolean
            Return String.Compare(lhs, rhs.Representation, True) = 0
        End Operator

        Public Shared Operator <>(ByVal lhs As String, ByVal rhs As Operation) As Boolean
            Return String.Compare(lhs, rhs.Representation, True) <> 0
        End Operator


        ''' <summary>
        ''' Parses an operator.
        ''' </summary>
        ''' <param name="op">The operator to parse.</param>
        ''' <exception cref="Expressions.ParseException">Thrown if the expression is not valid.</exception>
        ''' <returns>Returns the parsed operator.</returns>
        Public Shared Function Parse(ByVal op As String) As Operation
            Dim ret As Operation = Nothing
            If Not TryParse(op, ret) Then Throw New Expressions.ParseException(0)
            Return ret
        End Function

        ''' <summary>
        ''' Tries to parse an operator.
        ''' </summary>
        ''' <param name="op">The operator to parse.</param>
        ''' <param name="ret">The parsed operator.</param>
        ''' <returns>Returns whether parsing was successful.</returns>
        Public Shared Function TryParse(ByVal op As String, ByRef ret As Operation) As Boolean
            Dim t As Type = GetType(Operation)

            For Each m As Reflection.MemberInfo In t.FindMembers(Reflection.MemberTypes.Field, Reflection.BindingFlags.Public Or Reflection.BindingFlags.Static, Nothing, Nothing)
                Dim o As Operation = DirectCast(t.InvokeMember( _
                    m.Name, _
                    Reflection.BindingFlags.GetField Or Reflection.BindingFlags.Public Or Reflection.BindingFlags.Static, _
                    Nothing, Nothing, Nothing _
                ), Operation)
                If String.Compare(op, o.Representation, True) = 0 Then
                    ret = o
                    Return True
                End If
            Next

            Return False
        End Function

#End Region

#End Region

#Region " Private "

        Private ReadOnly Representation As String

#Region " Constructors "

        Private Sub New(ByVal representation As String, ByVal is_binary As Boolean)
            Me.Representation = representation
            Me.IsBinary = is_binary
        End Sub

#End Region

#End Region

    End Class

End Namespace