Imports madrat.Calculate.Parser

Namespace Calculate.Expressions

    Public Class PostfixExpression : Inherits Expression

#Region " Public "

#Region " Constructors "

        <DebuggerHidden()> _
        Public Sub New(ByVal expression As String, Optional ByVal start As Integer = 0)
            MyBase.New(expression, start)
        End Sub

#End Region

#Region " Methods "

        Public Overrides Function TryParse(ByRef ret As ParsedExpression) As Boolean
            Scanner = New PrefixPostfixScanner(Expression, Start)

            Dim tokens As New Stack ' This is needed so we get the reverse order.
            Stack = New Stack

            Do Until TypeOf Scanner.Lookahead Is Parser.EndOfStreamToken
                Select Case True
                    Case TypeOf Scanner.Lookahead Is Parser.OperationToken
                        Dim op As Operation = DirectCast(Scanner.Lookahead, Parser.OperationToken).Operation
                        tokens.Push(op)
                    Case TypeOf Scanner.Lookahead Is Parser.FunctionToken
                        Dim fun As [Function] = Functions.ResolveFunction(DirectCast(Scanner.Lookahead, Parser.FunctionToken).Function)
                        tokens.Push(fun)
                    Case TypeOf Scanner.Lookahead Is Parser.NumberToken
                        tokens.Push(DirectCast(Scanner.Lookahead, Parser.NumberToken).Number)
                    Case TypeOf Scanner.Lookahead Is Parser.VariableToken
                        tokens.Push(DirectCast(Scanner.Lookahead, Parser.VariableToken).Variable)
                    Case Else
                        [Error] = New ParseException(Scanner.Position, "An invalid symbol was entered.")
                        ret = Nothing
                        Return False
                End Select

                Scanner.Match(Scanner.Lookahead)
            Loop

            ' Reverse the order of the stack.
            Do Until tokens.Count = 0
                Stack.Push(tokens.Pop())
            Loop

            ret = New ParsedExpression(Expression, Stack)
            Return True
        End Function

#End Region

#End Region

    End Class

End Namespace