Namespace Calculate.Expressions

    Public Class Stack : Inherits Stack(Of StackToken)
        Implements Common.Cloneable(Of Stack)

#Region " Public "

#Region " Constructors "

        <DebuggerHidden()> _
        Public Sub New()

        End Sub

        <DebuggerHidden()> _
        Public Sub New(ByVal collection As IEnumerable(Of StackToken))
            MyBase.New(collection)
        End Sub

#End Region

#Region " Properties "

        Public Property Variables() As Dictionary(Of String, Values)
            <DebuggerHidden()> _
            Get
                Return _variables
            End Get
            <DebuggerHidden()> _
            Set(ByVal value As Dictionary(Of String, Values))
                _variables = value
            End Set
        End Property

#End Region

#Region " Methods "

        Public Overloads Sub Push(ByVal number As Double)
            Dim numbers As New Values()

            numbers.Add(number)
            MyBase.Push(New ListToken(numbers))
        End Sub

        Public Overloads Sub Push(ByVal list As Values)
            MyBase.Push(New ListToken(list))
        End Sub

        Public Overloads Sub Push(ByVal operation As Operation)
            MyBase.Push(New OperationToken(operation))
        End Sub

        Public Overloads Sub Push(ByVal [function] As [Function])
            MyBase.Push(New FunctionToken([function]))
        End Sub

        Public Overloads Sub Push(ByVal variable As String)
            MyBase.Push(New VariableToken(variable))
        End Sub

        Public Function Execute(ByVal params As Dictionary(Of String, Values)) As Values
            Dim account As New Stack(Of Values)

            Me.Params = params

            Do While Count > 0
                Dim tok As StackToken = Pop()

                Select Case True
                    Case TypeOf tok Is OperationToken
                        Dim op As Operation = DirectCast(tok, OperationToken).Operation

                        Select Case op.NumberOfOperands
                            Case 1
                                account.Push(op.Perform(account.Pop()))
                            Case 2
                                Dim v1 As Values = account.Pop()
                                Dim v2 As Values = account.Pop()
                                account.Push(op.Perform(v2, v1))
                            Case 3
                                Dim v1 As Values = account.Pop()
                                Dim v2 As Values = account.Pop()
                                Dim v3 As Values = account.Pop()
                                account.Push(op.Perform(v3, v2, v1))
                        End Select
                    Case TypeOf tok Is FunctionToken
                        Dim fun As [Function] = DirectCast(tok, FunctionToken).Function
                        Dim vals As New List(Of Values)

                        For i As Integer = 1 To fun.ArgumentNumber
                            vals.Add(account.Pop())
                        Next

                        vals.Reverse()

                        account.Push(fun.Perform(vals.ToArray()))
                    Case TypeOf tok Is NumberToken
                        Debug.Assert(False)
                    Case TypeOf tok Is ListToken
                        account.Push(DirectCast(tok, ListToken).Numbers)
                    Case TypeOf tok Is VariableToken
                        account.Push(ResolveVariable(tok.ToString()))
                End Select
            Loop

            If account.Count < 1 Then Throw New OverflowException("Account stack underflow")
            If account.Count > 1 Then Throw New OverflowException("Account stack overflow")
            Return account.Pop()
        End Function

        Public Function Clone() As Stack Implements Common.Cloneable(Of Stack).Clone
            Dim arr As StackToken() = ToArray()
            Array.Reverse(arr)
            Return New Stack(arr)
        End Function

        Public Function UntypedClone() As Object Implements System.ICloneable.Clone
            Return Clone()
        End Function

#End Region

#End Region

#Region " Private "

        Private _variables As Dictionary(Of String, Values)
        Private _params As Dictionary(Of String, Values)

#Region " Properties "

        Private Property Params() As Dictionary(Of String, Values)
            <DebuggerHidden()> _
            Get
                Return _params
            End Get
            <DebuggerHidden()> _
            Set(ByVal value As Dictionary(Of String, Values))
                _params = value
            End Set
        End Property

#End Region

#Region " Methods "

        Private Function ResolveVariable(ByVal var As String) As Values
            If Params.ContainsKey(var) Then Return Params(var)
            Return Variables(var)
        End Function

#End Region

#End Region

    End Class

End Namespace