Imports madrat.Calculate.Expressions

Namespace Calculate

    Public Class InputReader

#Region " Public "

        Public Enum ActionState
            Input
            Output
            Computation
        End Enum

        Public Enum DisplayPrecision
            Low
            High
        End Enum

#Region " Constructors "

        Public Sub New(Optional ByVal expression_format As ExpressionFormat = Expressions.ExpressionFormat.Infix)
            Me.ExpressionFormat = expression_format
            Console.TreatControlCAsInput = False
            AddHandler Console.CancelKeyPress, AddressOf CancelHandler
        End Sub

#End Region

#Region " Properties "

        Public Property ExpressionFormat() As ExpressionFormat
            <DebuggerHidden()> _
            Get
                Return _expression_format
            End Get
            <DebuggerHidden()> _
            Set(ByVal value As ExpressionFormat)
                _expression_format = value
            End Set
        End Property

        Public Property State() As ActionState
            <DebuggerHidden()> _
            Get
                Return _state
            End Get
            <DebuggerHidden()> _
            Set(ByVal value As ActionState)
                _state = value
            End Set
        End Property

        Public Property Precision() As DisplayPrecision
            <DebuggerHidden()> _
            Get
                Return _precision
            End Get
            <DebuggerHidden()> _
            Set(ByVal value As DisplayPrecision)
                _precision = value
            End Set
        End Property

#End Region

#Region " Methods "

        Public Sub Execute()
            Command.Info.Execute("")
            Utility.Display.WrapTextOut("Type ':help' for a help message!")
            ReadInputLoop()
        End Sub

        Public Function CalculateValues( _
            ByVal input As String, _
            Optional ByVal start As Integer = 0, _
            Optional ByVal format As ExpressionFormat = ExpressionFormat.Infix _
        ) As Values
            Try
                Dim cc As New System.ComponentModel.BackgroundWorker()
                Dim bc As New BackgroundComputer(AddressOf Expression.Parse(input, start, format).Execute)

                Do While bc.IsRunning
                    If State = ActionState.Input Then bc.Abort()
                Loop

                If bc.ReturnValue IsNot Nothing Then
                    Utility.Display.FormatResult(bc.ReturnValue)
                    Answers.Push(bc.ReturnValue)
                End If

                If bc.Error IsNot Nothing Then Throw bc.Error

                Return bc.ReturnValue
            Catch ex As ParseException
                Utility.Display.WrapTextOut(ex.ToString())
            Catch ex As OverflowException
                Utility.Display.WrapTextOut(ex.ToString())
            Catch ex As ArgumentOutOfRangeException
                Utility.Display.WrapLongText("Index out of range.")
            Catch ex As ArgumentException
                Utility.Display.WrapTextOut(ex.Message)
            Catch ex As ArithmeticException
                Utility.Display.WrapTextOut(ex.Message)
            Catch ex As InvalidOperationException
                ' This may only appear when using prefix or postfix mode because the parser does not
                ' check the number of parameters to a function.
                Utility.Display.WrapTextOut("You have entered too few parameters for a function call.")
            Catch ex As Exception
                Utility.Display.WrapTextOut("An unexpected exception occurred:")
                Utility.Display.WrapTextOut(ex.ToString())
            End Try

            Return Nothing
        End Function

#End Region

#End Region

#Region " Private "

        Private _expression_format As ExpressionFormat
        Private _state As ActionState
        Private _precision As DisplayPrecision

#Region " Methods "

        Private Sub CancelHandler(ByVal sender As Object, ByVal args As ConsoleCancelEventArgs)
            Select Case State
                Case ActionState.Input
                    args.Cancel = True
                Case ActionState.Output
                    Console.WriteLine("... cancelled.")
                    args.Cancel = True
                    State = ActionState.Input
                Case ActionState.Computation
                    Console.WriteLine("... cancelled.")
                    args.Cancel = True
                    State = ActionState.Input
            End Select
        End Sub

        Private Sub ReadInputLoop()
            Do
                State = ActionState.Input
                Console.Write("> ")
                Dim input As String = Nothing
                Do While input Is Nothing
                    input = Console.ReadLine()
                Loop
                Dim scanner As New Parser.WordScanner(input)

                ' Parsing ...

                Try
                    If TypeOf scanner.Lookahead Is Parser.CommandToken Then
                        Dim cmd As Command = DirectCast(scanner.Lookahead, Parser.CommandToken).Command
                        scanner.Match(Of Parser.CommandToken)()
                        cmd.Execute(input)
                        If cmd = Command.Quit Then Exit Do
                    ElseIf input.Trim().Length > 0 Then
                        ' Regular expression.
                        CalculateValues(input, 0, ExpressionFormat)
                    End If
                Catch ex As ParseException
                    If ex.position = 0 Then
                        ' There has been an invalid command.
                        Utility.Display.WrapTextOut("Invalid command. Enter ':help' for an overview over the commands. ")
                        Utility.Display.WrapTextOut("Type ':help <cmd>' for detailed information about 'cmd'.")
                    Else
                        Utility.Display.WrapTextOut( _
                            "The input was invalid. Enter ':help' for information about valid statements." _
                        )
                    End If
                End Try
            Loop
        End Sub

#End Region

#End Region

    End Class

End Namespace