Namespace Calculate.Parser

    Public Class InfixScanner : Inherits Scanner

        Private ReadOnly Tokens As String() = New String() { _
            "Command", "Const", "Variable", "Function", "Number", _
            "WS", "Op", "OBR", "CBR", "Comma", "Assign", "Error" _
        }

        Public Sub New(ByVal input As String, Optional ByVal start As Integer = 0)
            MyBase.New(input, start)
        End Sub

        Protected Overrides Function GetTokenFromMatch(ByVal match As Text.RegularExpressions.Match) As Token
            For Each find As String In Tokens
                If match.Groups(find).Success Then
                    Select Case find
                        Case "Command"
                            Return New CommandToken(match.Value)
                        Case "Const"
                            Return New NumberToken(ResolveConstant(match.Value))
                        Case "Variable"
                            Return New VariableToken(match.Value)
                        Case "Function"
                            Return New FunctionToken(match.Value)
                        Case "Number"
                            Return New NumberToken(match.Value)
                        Case "WS"
                            Return Nothing
                        Case "Op"
                            Return New OperationToken(match.Value)
                        Case "OBR"
                            Return New OpenBraceToken()
                        Case "CBR"
                            Return New CloseBraceToken()
                        Case "Comma"
                            Return New CommaToken()
                        Case "Assign"
                            Return New AssignToken()
                        Case "Error"
                            Return New ErrorToken(Position)
                    End Select
                End If
            Next

            ' This statement should never be reached.
            Return New ErrorToken(Position)
        End Function

        Protected Overrides Function SearchPattern() As String
            Return _
                "(?<Command>^:[a-z]+)(\b|$)|" & _
                "(?<Const>p|e|\u03C0|\u212F)\b|" & _
                "(?<Variable>\$[a-z][a-z0-9]*)|" & _
                "(?<Function>[a-z][a-z0-9]*)|" & _
                "(?<Number>\d+(\.\d+)?(E[-+]?\d+)?)|" & _
                "(?<WS>(\t| )+)|" & _
                "(?<Op>\*\*|<<|>>|\.\.|[-+|^*/\\%&?\!;:])|" & _
                "(?<OBR>\()|" & _
                "(?<CBR>\))|" & _
                "(?<Comma>,)|" & _
                "(?<Assign>=)|" & _
                "(?<Error>.+)"
        End Function

        <DebuggerHidden()> _
        Private Function ResolveConstant(ByVal input As String) As Double
            Select Case input.ToLower()
                Case "p", ChrW(&H3C0)
                    Return Math.PI
                Case "e", ChrW(&H212F)
                    Return Math.E
                Case Else
                    Return Double.NaN
            End Select
        End Function

    End Class

End Namespace